import math
import collections


def namedtuple(obj):
    if isinstance(obj, dict):
        keys = []
        values = []
        for key, value in obj.iteritems():
            keys.append(key)
            values.append(namedtuple(value))
        return collections.namedtuple("object", keys)(*values)
    elif hasattr(obj, "_asdict"):
        return namedtuple(obj._asdict())
    elif isinstance(obj, (set, list, tuple)):
        array = []
        for item in obj:
            array.append(namedtuple(item))
        return array
    else:
        return obj


class Page(object):
    __slots__ = "page", "pages", "current"

    def __init__(self, page, pages, current):
        self.page = page
        self.current = current
        self.pages = pages


class Pagination(object):
    __slots__ = "ipp", "fpp"

    def __init__(self, per_page):
        self.ipp = per_page
        self.fpp = float(per_page)

    def transform(self, page_no):
        skip = (page_no - 1) * self.ipp
        return skip, self.ipp

    def pages(self, items_count):
        return int(math.ceil(items_count / self.fpp))

    def wrap(self, items, items_count, page_no=None):
        return Page(items, self.pages(items_count), page_no)