import math
import collections


def namedtuple(obj):
    if isinstance(obj, dict) or hasattr(obj, "iteritems"):
        keys = []
        values = []
        for key, value in obj.iteritems():
            keys.append(key)
            values.append(namedtuple(value))
        return collections.namedtuple("object", keys)(*values)
    elif isinstance(obj, (set, list, tuple)):
        array = []
        for item in obj:
            array.append(namedtuple(item))
        return array
    else:
        return obj


class Page(object):
    __slots__ = "page", "pages", "current"

    def __init__(self, page, pages, current):
        self.page = page
        self.current = current
        self.pages = pages

    def get_segment(self, segment_size=10):
        page_no = self.current
        pages = self.pages

        view_min = min(page_no - page_no % segment_size + (page_no / segment_size == 0), pages)
        view_max = min(view_min + segment_size + 1, pages)

        return xrange(view_min, view_max + (view_max == pages))

    def get_prev_segment(self, segment_size=10):
        page_no = self.current
        value = (page_no / segment_size - 1) * segment_size + segment_size - 1
        return None if value < 1 else value

    def get_next_segment(self, segment_size=10):
        page_no = self.current
        value = (page_no / segment_size + 1) * segment_size + 1 + (page_no / segment_size == 0)
        return None if value >= self.pages else value

    def __iter__(self):
        return iter(self.page)


class Pagination(object):
    __slots__ = "per_page", "_fpp"

    def __init__(self, per_page):
        self.per_page = per_page
        self._fpp = float(per_page)

    def transform(self, page_no):
        skip = (page_no - 1) * self.per_page
        return skip, self.per_page

    def pages(self, items_count):
        return int(math.ceil(items_count / self._fpp))

    def wrap(self, items, items_count, page_no=1):
        return Page(items, self.pages(items_count), page_no)