__author__ = "ardevelop"

import os
import csv


class Dictionary:
    def __init__(self, path):
        self._languages = {}
        self._selected_dict = {}
        self._selected = None

        if not os.path.isdir(path):
            raise Exception("Invalid i18n path")

        files = [([], os.path.join(path, item)) for item in os.listdir(path)]

        while files:
            origin, filename = files.pop(0)
            if os.path.isdir(filename):
                files.extend([(origin + [os.path.basename(filename)], os.path.join(filename, item)) for item in
                              os.listdir(filename)])
            else:
                if item.endswith(".csv"):
                    language = item[:-4]
                    with open(filename, "r") as fp:
                        for row in csv.reader(fp):
                            if row and len(row) > 1:
                                key = row[0].strip(" \t")
                                if origin:
                                    key = ".".join(origin) + "." + key
                                try:
                                    self._languages[language][key] = row[1]
                                except KeyError:
                                    self._languages[language] = {key: row[1]}

    @property
    def selected(self):
        return self._selected

    def select(self, language):
        try:
            self._selected_dict = self._languages[language]
            self._selected = language
        except KeyError:
            self._selected_dict = None

    def translate(self, text):
        try:
            return self._selected_dict[text]
        except (KeyError, TypeError):
            return text