__author__ = "ardevelop"

NO_DEFAULT = object()
REQUIRED = "validation.required"
INVALID = "validation.invalid"


class Validator:
    def __init__(self, i18n=None):
        self.i18n = i18n

    def validate(self, value, modifier=None, validators=None, default=NO_DEFAULT, required=False):
        if value is None or "" == value:
            if default != NO_DEFAULT:
                value = default
            elif required:
                return False, value, self.i18n.translate(REQUIRED) if self.i18n else REQUIRED
            else:
                value = None

        if value is not None and modifier:
            try:
                value = modifier(value)
            except ValueError:
                return False, value, self.i18n.translate(INVALID) if self.i18n else INVALID

        if validators:
            for validator in validators:
                valid, message = validator(value)
                if not valid:
                    return False, value, message

        return True, value, None